﻿using System;
using System.Linq;

using CMS.Scheduler;
using CMS.EventLog;
using CMS.Synchronization;
using CMS.Base;
using CMS.SiteProvider;
using CMS;
using CMS.DataEngine;

[assembly: RegisterCustomClass("Custom.StagingScheduledTask", typeof(Custom.StagingScheduledTask))]
namespace Custom
{
    /// <summary>
    /// Summary description for CustomTask
    /// </summary>
    public class StagingScheduledTask : ITask
    {
        /// <summary>
        /// Executes the task.
        /// </summary>
        /// <param name="ti">Info object representing the scheduled task</param>
        public string Execute(TaskInfo ti)
        {
            //Get the SiteInfo
            var siteInfo = SiteInfoProvider.GetSiteInfo(ti.CurrentSiteName);
            var source = string.Format("CustomContentStaging - {0}", siteInfo.DisplayName);
            var processedTasks = 0;
            // Get the staging server - replace TargetServer with your server code name
            ServerInfo targetServer = ServerInfoProvider.GetServerInfo("TargetServer", SiteContext.CurrentSiteID);
            //return string from the sync run
            var syncResult = "";
            bool isValid = false;


            if (targetServer != null)
            {
                // Get all staging tasks that target the given server
                var tasks = StagingTaskInfoProvider.SelectTaskList(SiteContext.CurrentSiteID, targetServer.ServerID, null, null);

                if (tasks.Any())
                {
                    //initialize helper for sync
                    StagingTaskRunner str = new StagingTaskRunner(targetServer.ServerID, siteInfo.SiteID);

                    foreach (var task in tasks)
                    {
                        //flag to set what tasks to sync
                        isValid = task.TaskType.IsOneOf(
                              //page actions
                              TaskTypeEnum.CreateDocument
                            , TaskTypeEnum.UpdateDocument
                            , TaskTypeEnum.DeleteDocument
                            , TaskTypeEnum.ArchiveDocument
                            , TaskTypeEnum.MoveDocument
                            , TaskTypeEnum.PublishDocument
                            , TaskTypeEnum.RejectDocument
                            , TaskTypeEnum.DeleteAllCultures
                            , TaskTypeEnum.BreakACLInheritance
                            , TaskTypeEnum.RestoreACLInheritance

                            //media library folder actions
                            , TaskTypeEnum.CopyMediaFolder
                            , TaskTypeEnum.CreateMediaFolder
                            , TaskTypeEnum.DeleteMediaFolder
                            , TaskTypeEnum.DeleteMediaRootFolder
                            , TaskTypeEnum.MoveMediaFolder
                            , TaskTypeEnum.RenameMediaFolder

                            //general object actions
                            , TaskTypeEnum.CreateObject
                            , TaskTypeEnum.UpdateObject
                            , TaskTypeEnum.DeleteObject
                            , TaskTypeEnum.AddToSite
                            , TaskTypeEnum.RemoveFromSite
                            );

                        // sample for processing only media files and media folder related actions
                        //isValid = task.TaskType.IsOneOf(
                        //          TaskTypeEnum.CreateObject
                        //        , TaskTypeEnum.UpdateObject
                        //        , TaskTypeEnum.DeleteObject

                        //        , TaskTypeEnum.CopyMediaFolder
                        //        , TaskTypeEnum.CreateMediaFolder
                        //        , TaskTypeEnum.DeleteMediaFolder
                        //        , TaskTypeEnum.DeleteMediaRootFolder
                        //        , TaskTypeEnum.MoveMediaFolder
                        //        , TaskTypeEnum.RenameMediaFolder
                        //        )
                        //    && (task.TaskObjectType == "media.file" || task.TaskObjectType == "media.folder");

                        if (isValid)
                        {
                            processedTasks++;

                            // Synchronizes the processed staging task to the target server
                            syncResult = str.RunSynchronization(task.TaskID);

                            if (string.IsNullOrEmpty(syncResult))
                            {
                                return string.Format("Scheduled task executed successfully. Processed {0} staging tasks out of total {1}", processedTasks, tasks.Count);
                            }
                            else
                            {
                                return string.Format("Scheduled task executed. Staging tasks failed with error: {0}", syncResult.ToString());
                            }
                        }
                        else
                        {
                            return string.Format("Scheduled task executed but the task type is not supported: {0}; Object type of: {1}", task.TaskType, task.TaskObjectType);
                        }
                    }
                    return null;
                }
                else
                {
                    return "No staging tasks found";
                }
            }
            else
            {
                return "Target server not found";
            }
            
        }
    }

    public static class EnumExtensions
    {
        public static bool IsOneOf(this Enum enumeration, params Enum[] enums)
        {
            return enums.Contains(enumeration);
        }
    }
}
